import logging
from looker_sdk import models40 as models, error
import coloredlogs
from tqdm import tqdm

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')
logging.getLogger("requests").setLevel(logging.WARNING)


class CreateInstanceLooks():
    def __init__(self, folder_mapping, sdk, content_metadata):
        self.sdk = sdk
        self.content_metadata = content_metadata
        self.folder_mapping = folder_mapping

    def create_query(self, look_metadata: dict) -> int:
        '''create a query from look metadata and return the id'''
        query_body = models.WriteQuery(
            model=look_metadata['query_obj']['model'] if look_metadata['query_obj']['model'] else None,
            view=look_metadata['query_obj']['view'] if look_metadata['query_obj']['view'] else None,
            fields=look_metadata['query_obj']['fields'] if look_metadata['query_obj']['fields'] else None,
            pivots=look_metadata['query_obj']['pivots'] if look_metadata['query_obj']['pivots'] else None,
            fill_fields=look_metadata['query_obj']['fill_fields'] if look_metadata['query_obj']['fill_fields'] else None,
            filters=look_metadata['query_obj']['filters'] if look_metadata['query_obj']['filters'] else None,
            filter_expression=look_metadata['query_obj']['filter_expression'] if look_metadata['query_obj']['filter_expression'] else None,
            sorts=look_metadata['query_obj']['sorts'] if look_metadata['query_obj']['sorts'] else None,
            limit=look_metadata['query_obj']['limit'] if look_metadata['query_obj']['limit'] else None,
            column_limit=look_metadata['query_obj']['column_limit'] if look_metadata['query_obj']['column_limit'] else None,
            total=look_metadata['query_obj']['total'] if look_metadata['query_obj']['total'] else None,
            row_total=look_metadata['query_obj']['row_total'] if look_metadata['query_obj']['row_total'] else None,
            subtotals=look_metadata['query_obj']['subtotals'] if look_metadata['query_obj']['subtotals'] else None,
            vis_config=look_metadata['query_obj']['vis_config'] if look_metadata['query_obj']['vis_config'] else None,
            filter_config=look_metadata['query_obj']['filter_config'] if look_metadata['query_obj']['filter_config'] else None,
            visible_ui_sections=look_metadata['query_obj']['visible_ui_sections'] if look_metadata['query_obj']['visible_ui_sections'] else None,
            dynamic_fields=look_metadata['query_obj']['dynamic_fields'] if look_metadata['query_obj']['dynamic_fields'] else None,
            query_timezone=look_metadata['query_obj']['query_timezone'] if look_metadata['query_obj']['query_timezone'] else None
        )
        response = self.sdk.create_query(body=query_body)
        return response
    def create_look(self, query_id: int, look_metadata: dict, folder_mapping: dict) -> dict:
        legacy_fid = look_metadata.get('legacy_folder_id')
        look_body = models.WriteLookWithQuery(
            title=look_metadata.get('title'),
            description=look_metadata['description'],
            query_id=query_id,
            folder_id=folder_mapping.get(legacy_fid) if folder_mapping.get(legacy_fid) != 'Shared' else '1') 
        response = self.sdk.create_look(body=look_body)
        return response

    def execute(self) -> dict:
        look_mapping = []

        for look in tqdm(self.content_metadata, desc="Look Creation", unit=" attributes", colour="#2c8558"):
            query = self.create_query(look_metadata=look)
            new_look = self.create_look(
                query_id=query.id, look_metadata=look, folder_mapping=self.folder_mapping)
            temp = {}
            temp['look_mapping'] = {}
            temp['look_mapping'][look.get('look_id')] = new_look.get('id')
            temp['folder_mapping'] = {}
            temp['folder_mapping'][look.get('legacy_folder_id')] = self.folder_mapping.get(
                look.get('legacy_folder_id'))
            look_mapping.append(temp)

        return look_mapping



import logging
import coloredlogs
import looker_sdk
import ruamel.yaml
from lmanage.capturator.user_group_capturation import role_config as rc
from lmanage.capturator.user_attribute_capturation import capture_ua_permissions as cup
from lmanage.capturator.folder_capturation import folder_config as fc, create_folder_yaml_structure as cfp
from lmanage.capturator.content_capturation import dashboard_capture as dc, look_capture as lc, board_capture as bc
from lmanage.utils import looker_object_constructors as loc, errorhandling as eh

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def main(**kwargs):

    ini_file = kwargs.get("ini_file")
    yaml_path = kwargs.get("yaml_export_path")
    yaml_path = yaml_path.split('.')[0]

    logger.info('creating yaml configuration file')

    if ini_file:
        sdk = looker_sdk.init40(config_file=ini_file)
    else:
        sdk = looker_sdk.init40()
    if eh.user_authentication_test(sdk=sdk):
        logger.info('User is successfully authenticated to the API')
    else:
        raise Exception(
            "User is not successfully authenticated please verify credentials")

    yaml = ruamel.yaml.YAML()
    yaml.register_class(loc.LookerFolder)
    yaml.register_class(loc.LookerModelSet)
    yaml.register_class(loc.LookerPermissionSet)
    yaml.register_class(loc.LookerRoles)
    yaml.register_class(loc.LookerUserAttribute)
    yaml.register_class(loc.LookObject)
    yaml.register_class(loc.DashboardObject)
    yaml.register_class(loc.BoardObject)

###############################################################
# Capture Folder Config #######################################
###############################################################
    folder_returns = fc.CaptureFolderConfig(sdk=sdk).execute()
    folder_structure_list = folder_returns[1]

    with open(f'{yaml_path}.yaml', 'w') as file:
        fd_yml_txt = '''# FOLDER_PERMISSIONS\n# Opening Session Welcome to the Capturator, this is the Folder place\n# -----------------------------------------------------\n\n'''
        file.write(fd_yml_txt)
        yaml.dump(folder_structure_list, file)

    folder_root_dict = folder_returns[0]

###############################################################
# Capture Roles ###############################################
###############################################################
    roles = rc.ExtractRoleInfo(sdk=sdk)

    looker_permission_sets = roles.extract_permission_sets()
    looker_model_sets = roles.extract_model_sets()

    looker_roles = roles.extract_role_info()
    with open(f'{yaml_path}.yaml', 'a') as file:
        r_yml_txt = '''# Looker Role\n# Opening Session Welcome to the Capturator, this is the Role place\n# -----------------------------------------------------\n\n'''
        file.write(r_yml_txt)

        file.write('\n\n# PERMISSION SETS\n')
        yaml.dump(looker_permission_sets, file)
        file.write('\n\n# MODEL SETS\n')
        yaml.dump(looker_model_sets, file)
        file.write('\n\n# LOOKER ROLES\n')
        yaml.dump(looker_roles, file)
###############################################################
# Capture User Attributes #####################################
###############################################################
    looker_ua = cup.ExtractUserAttributes(sdk=sdk).create_user_attributes()
    yaml.register_class(cup.LookerUserAttribute)
    with open(f'{yaml_path}.yaml', 'a') as file:
        file.write('\n\n# USER_ATTRIBUTES\n')
        yaml.dump(looker_ua, file)

###############################################################
# Capture Users Content #######################################
###############################################################
    # Capture Content Boards
    # boards = bc.CaptureBoards(sdk=sdk).execute()
    # with open(f'{yaml_path}_content.yaml', 'w+') as file:
    #     file.write('\n\n# BoardData\n')
    #     yaml.dump(boards, file)

    # Capture Look Content
    looks = lc.CaptureLookObject(
        sdk=sdk, folder_root=folder_root_dict).execute()
    with open(f'{yaml_path}_content.yaml', 'a') as file:
        file.write('\n\n# LookData\n')
        yaml.dump(looks, file)

    # Capture Dashboard Content
    dash_content = dc.CaptureDashboards(
        sdk=sdk, folder_root=folder_root_dict).execute()
    with open(f'{yaml_path}_content.yaml', 'a') as file:
        fd_yml_txt = '\n\n# Dashboard Content\n'
        file.write(fd_yml_txt)
        yaml.dump(dash_content, file)

    # FIND UNIQUE USER ATTRIBUTES AND ATTRIBUTE TO TEAM
    logger.info('\n\n\n Lmanage has finished capturing your Looker instance!\n')
    logger.info('\n\nplease find captured settings metadata at: \n%s.yaml \n\ncaptured content metadata at:\n%s_content.yaml', yaml_path, yaml_path)


if __name__ == "__main__":
    instance = 'dev'
    IP = (
        f'/usr/local/google/home/hugoselbie/code_sample/py/ini/{instance}.ini')
    YP = (
        f'/usr/local/google/home/hugoselbie/code_sample/py/lmanage/tests/example_yamls/{instance}_output.yaml')

    main(yaml_export_path=YP, ini_file=IP)



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import json
from lmanage.collections import defaultdict
import configparser as ConfigParser
import re
from lmanage.itertools import chain

import coloredlogs
import looker_sdk
from looker_sdk import models
import pandas as pd
from lmanage.utils import parsing_sql
from lmanage.utils import create_df

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


def find_model_files(proj):
    """Fetches model files from lmanage.PyLookML obj.
    Scans all files in PyLookML object and returns a file
    if that file has a type of Model.

    Args:
        proj: The project from lmanage.PyLookML
    Returns:
        The unparsed model file of a project.
    """
    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'model':
            return file


def get_view_path(proj):
    """Fetches the file path of a PyLookML Oject.

    Iterates over a list of PyLookML files and returns a list of the files
    if they are of type 'View'
    Args:
        proj: The project from lmanage.PyLookML
    Returns:
        A list of all the file paths pointing towards view objects that are
        kept in a PyLookML object.
        For example:
        ['proj/views/order_items.view.lkml',
        'proj/views/inventory_items.view.lkml',
        'proj/views/products.view.lkml']
    """
    view_list = defaultdict(list)
    for file in proj.files():
        path = file.path
        path = path.split('.')
        if path[-2] == 'view':
            name = file.name.split('.')
            view_list[name[0]].append(file.path)

    return view_list


def fetch_view_files(proj):
    """Fetches the all the named view objects listed in explores of a
    Model file specifically.
    Identifies a model file, iterates over all the explore objects in the
    model file, appends their base view and joined views to a dict object
    Args:
        proj: The project from lmanage.PyLookML
    Returns:
        A dict with key: being the base view and values: list of all the
        used views in the explore object.
        For example:
         defaultdict(<class 'list'>,
         {'order_items':
            ['order_items',
             'order_facts',
             'inventory_items'],
         'events':
            ['events',
             'sessions',
             'session_landing_page']})"""

    true_view_names = defaultdict(list)

    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'model':
            my_model = file

            for explore in my_model.explores:
                if 'view_name' not in explore and 'from' not in explore:
                    true_view_names[explore.name].append(explore.name)
                if 'view_name' in explore:
                    true_view_names[explore['name']].append(
                        explore['view_name'].value)
                if 'from' in explore:
                    true_view_names[explore['name']].append(
                        explore['from'].value)
                if 'join' in explore:
                    for join in explore['join']:
                        true_view_names[explore['name']].append(join['name'])
                        if 'view_name' in join.__dict__.keys():
                            true_view_names[explore['name']].append(
                                join['view_name'].value)
                        if 'from' in join.__dict__.keys():
                            true_view_names[explore['name']].append(
                                join['from'].value)
    return true_view_names


def get_sql_table_name(proj):
    """Fetches the sql_table_name of view files listed in a PyLookML Oject.

    Iterates over a list of PyLookML files identifies view files aand returns a
    list of the sql_table_names if they exist in an object of type 'View'
    Args:
        proj: The project from lmanage.PyLookML
    Returns:
        A list of all the sql_table_names that are found in a series of view
        objects.
        For example:
        ['public.order_items','public.inventory_items','public.events']
    """
    view_list = defaultdict(list)

    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'partial_model':
            for view in myFile.views:
                if view.sql_table_name.value:
                    view_list[path].append(view.sql_table_name.value)

    return view_list


def get_sql_table_name_list(sql_table_name_dict, key: True):
    response = []
    for k, v in sql_table_name_dict.items():
        if key:
            response.append(k)
        else:
            response.append(v)

    return response if key else list(chain(*response))


def parse_sql(sdk, qid: int):
    """Idenfies the base tables and joins used by a Looker query.

    Iterates over a list of PyLookML files identifies view files aand returns a
    list of the sql_table_names if they exist in an object of type 'View'
        qid: (int) query_id from lmanage.a  Looker query
        (n.b. NOT THE SAME AS A QID in the url)
    Returns:
        A list of all the tables that are found from lmanage.a Looker generated
        SQL query.
        For example:
        ['public.order_items','public.inventory_items','public.events']
    Exception:
        If a query is broken for whatever reason an Exception is raised to
        continue the program running
    """
    try:
        sql_response = sdk.run_query(query_id=qid, result_format='sql')
        if type(sql_response) == str:
            tables = parsing_sql.extract_tables(sql_response)
            return tables
        else:
            return sql_response
    except looker_sdk.error.SDKError:
        return('No Content')


def get_sql_from_elements(sdk, content_results):
    """Amends returned SDK System__Activity reponse with sql tables used
    from lmanage.the `parse_sql` function.

    Iterates over the response from lmanage.get_dashboards and runs the parse_sql
    function for each returned dashboard element, returns the list of tables
    and amends the dict response and returns it
    Args:
        sdk: Looker SDK object
        content_results: (dict) response from lmanage.get_dashboards function call
    Returns:
        An amended dict response with the sql columns used by each element
        extracted our of the Looker generated SQL for each dashboard object.
        For example:
    [{'dashboard.id': 1,
         'dashboard_element.id': 1,
         'dashboard_element.type': 'vis',
         'dashboard_element.result_source': 'Lookless',
         'query.model': 'bq',
         'query.view': 'order_items',
         'query.formatted_fields': '["order_items.count"]',
         'query.id': 59,
         'dashboard.title': 'dash_1',
         'look.id': None,
         'sql_joins': ['`looker-private-demo.ecomm.order_items`']}]
    """
    for dash in content_results:
        query_id = dash['query.id']
        sql_value = parse_sql(sdk, query_id)

        dash['sql_joins'] = sql_value

    return content_results


def get_dashboards(sdk):
    """Uses the Looker SDK System__Activity model to extract dashboard
    and dashboard_element metadata.

    Simple run_inline_query call to Looker SDK
    Args:
        sdk: Looker SDK object
    Returns:
        An dict response with the dashboard and dashboard_element metadata.
        For example:
    [{'dashboard.id': 1,
         'dashboard_element.id': 1,
         'dashboard_element.type': 'vis',
         'dashboard_element.result_source': 'Lookless',
         'query.model': 'bq',
         'query.view': 'order_items',
         'query.formatted_fields': '["order_items.count"]',
         'query.id': 59,
         'dashboard.title': 'dash_1',
         'look.id': None}]
    """
    query_config = models.WriteQuery(
        model="system__activity",
        view="dashboard",
        fields=[
            "dashboard.id",
            "dashboard_element.id",
            "dashboard_element.type",
            "dashboard_element.result_source",
            "query.model",
            "query.view",
            "query.formatted_fields",
            "query.id",
            "dashboard.title",
            "look.id"
        ],
        filters={
            "dashboard_element.type": "-text",
            "dashboard.deleted_date": "NULL"
        },
        limit='5000'
    )
    query_response = sdk.run_inline_query(
        result_format='json',
        body=query_config
    )

    query_response = json.loads(query_response)

    return query_response


def test_period_appearence(input_response):
    """Checks existence of a period in an input string.

    Simple return True if a period is detected in a string
    Args:
        input_response: (str)
    Returns:
        A boolean response
    For example:
    True or False
    """
    test_period = re.search(r"\.", input_response)
    return bool(test_period)


def match_join_per_query(myresults):
    result = []
    sql_join = myresults['sql_joins']
    sql_table_name = myresults['sql_table_name']

    for sql in sql_join:
        if not bool(test_period_appearence(sql)):
            result.append(sql)
        for name in sql_table_name:
            if sql == name:
                result.append(sql)
    myresults['used_joins'] = result
    return myresults


def all_joins(myresults):
    ''' function returns all used joins for a given set of content '''
    result = []
    for element in range(0, len(myresults)):
        sql_join = myresults[element]['sql_joins']
        sql_table_name = myresults[element]['sql_table_name']

        for sql in sql_join:
            if not bool(test_period_appearence(sql)):
                result.append(sql)
            for name in sql_table_name:
                if sql == name:
                    result.append(sql)
        myresults[element]['used_joins'] = result
    return myresults


def all_views(myresults, proj):
    result = []
    for element in range(0, len(myresults)):
        used_joins = myresults[element]['used_joins']
        for join in used_joins:
            if not bool(test_period_appearence(join)):
                result.append(join)
        for file in proj.files():
            path = file.path
            myFile = proj.file(path)
            if myFile.type != 'model':
                for view in myFile.views:
                    if view.sql_table_name.value in used_joins:
                        result.append(view.name)

    myresults[element]['used_view_names'] = result
    return myresults


def match_views_per_query(myresults, proj):
    result = []
    paths = []
    used_joins = myresults['used_joins']
    for join in used_joins:
        if not bool(test_period_appearence(join)):
            result.append(join)
    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type != 'model':
            for view in myFile.views:
                if view.sql_table_name.value in used_joins:
                    result.append(view.name)
                    paths.append(path)

    myresults['used_view_names'] = result
    myresults['sql_table_paths_'] = paths
    return myresults


def find_unused_views(myresults):
    used_view_names = sorted(myresults['used_view_names'])
    potential_joins = sorted(myresults['potential_join'])
    result = potential_joins

    for view in used_view_names:
        if view in potential_joins:
            result.remove(view)
    myresults['unused_joins'] = result
    return myresults


def match_view_to_dash(content_results, explore_results, sql_table_name, proj):
    tables_in_explore = []

    sql_table_name_ = get_sql_table_name_list(sql_table_name, key=False)

    for content in content_results:
        result = defaultdict(list)
        result['dashboard_id'] = content['dashboard.id']
        result['element_id'] = content['dashboard_element.id']
        result['sql_joins'] = content['sql_joins']
        result['fields_used'] = content['query.formatted_fields']

        result['sql_table_name'] = sql_table_name_

        for explore, tables in explore_results.items():
            if content['query.view'] == explore:
                result['potential_join'] = tables
                tables_in_explore.append(result)
    return tables_in_explore


def main(**kwargs):
    ini_file = kwargs.get("ini_file")
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(ini_file)
    project_repo = kwargs.get("project")
    logger.info(f'your project repo is at {project_repo}')
    file_path = kwargs.get("path")
    logger.info(f'your output file is at {file_path}')
    table_mask = kwargs.get("table")
    field_mask = kwargs.get("field")

    create_df.check_ini(ini_file)

    sdk = looker_sdk.init31(config_file=ini_file)
    logger.info

    project = 'legacy_tool_replacing_with_lkml_parser'

    content_results = get_dashboards(sdk)
    logger.debug(f'dashboard_response = {content_results}')
    db_response = get_sql_from_elements(sdk, content_results)
    logger.debug(f'db_response = {db_response}')
    explore_results = fetch_view_files(proj=project)
    logger.debug(f'explore results = {explore_results}')

    sql_table_names = get_sql_table_name(proj=project)
    logger.debug(f'sql_table_names = {sql_table_names}')

    combine = match_view_to_dash(
        db_response, explore_results, sql_table_names, proj=project)
    logger.debug(f'combine = {combine}')

    for element in range(0, len(combine)):
        match_join_per_query(combine[element])
        match_views_per_query(combine[element], project)
        find_unused_views(combine[element])

    df = pd.DataFrame(combine)
    del df['sql_table_name']
    del df['potential_join']
    del df['sql_joins']

    if table_mask is None and field_mask is None:
        logger.info('you have not set any field or table filters')

        df.to_csv(f'{file_path}')
        logger.info(df)

    elif table_mask is not None:
        logger.info(f'your table filter = {table_mask}')

        mask = df['used_view_names'].apply(lambda x: table_mask in x)
        df = df[mask]
        df.to_csv(f'{file_path}')

        logger.info(df)

    elif field_mask is not None:
        logger.info(f'your field filter = {field_mask}')

        mask = df['fields_used'].apply(lambda x: field_mask in x)



import json
import yaml 
import logging
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class Yaml:
    def __init__(self, yaml_path):
        self.yaml_path = yaml_path
        with open(self.yaml_path, 'r') as file:
            self.parsed_yaml = yaml.load(file, Loader=yaml.BaseLoader)

    def get_permission_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'permissions' in list(objects.keys()):
                response.append(objects)
        response = [r for r in response if r.get('name') != 'Admin']
        return response

    def get_model_set_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'models' in list(objects.keys()):
                response.append(objects)
        return response

    def get_role_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'permission_set' and 'model_set' in list(objects.keys()):
                response.append(objects)
        response = [r for r in response if r.get('name') != 'Admin']
        return response

    def get_folder_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'content_metadata_id' in list(objects.keys()):
                response.append(objects)
        return response

    def get_user_attribute_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'hidden_value' in list(objects.keys()):
                response.append(objects)
        return response

    def get_look_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'query_obj' in list(objects.keys()):
                response.append(objects)
        return response
    
    def get_dash_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'lookml' in list(objects.keys()):
                response.append(objects)
        return response

    def get_board_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'board_sections' in list(objects.keys()):
                response.append(objects)
        return response



import ast
import logging
import coloredlogs
from looker_sdk import error
from time import sleep
import random

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


def calc_done_percent(iterator: int, total: int) -> str:
    percent = (iterator/total) * 100
    return f'{int(percent)} %'


def return_error_message(input: str):
    # err_response = input.args[0]
    # err_response = json.loads(err_response)
    if len(input.errors) == 0:
        err_response = input.message
    else:
        err_response = input.errors[0].message

    return err_response

def user_authentication_test(sdk) -> bool:
    try:
        sdk.me()
        return True
    except error.SDKError():
        return False
    
def return_sleep_message(call_number=0, quiet=False):
    call_number = call_number+1
    sleep_number = 2 ** call_number
    sleep(sleep_number)
    if not quiet:
        logger.warn(f'Looker is overwhelmed sleeping for {sleep_number} secs')


def dedup_list_of_dicts(input: list):
    stringinput = list(set([str(x) for x in input]))
    deduped = [ast.literal_eval(x) for x in stringinput]
    return deduped


def counter(i=0):
    i += 1
    return i



import logging
from looker_sdk import models, error
import coloredlogs
from tqdm import tqdm

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("requests").setLevel(logging.WARNING)


class CreateInstanceFolders():
    def __init__(self, folder_metadata, sdk):
        self.sdk = sdk
        self.folder_metadata = folder_metadata

    def get_all_folders(self):
        instance_folders = self.sdk.all_folders()
        response = {folder.id: folder.name for folder in instance_folders}
        return response

    def scour_folder(self):
        exempt_folders = ['1', '2', '3', '4', 'lookml']
        folders = self.get_all_folders()
        error_count = 0
        #allowed_folders = [folder_obj for folder_obj in folders.keys() if folder_obj not in exempt_folders]
        while error_count < 1:
            folders = self.get_all_folders()
            x = [folder_obj for folder_obj in folders.keys(
            ) if folder_obj not in exempt_folders]
        #for fid in tqdm(allowed_folders):
            try:
                self.sdk.delete_folder(folder_id=x[0])
                logger.debug('deleting folder %s to start afresh',
                            folders.get(x[0]))
            except error.SDKError as e:
                logger.error(e)
                error_count = 1

    def unnest_folder_data(self, folder_data):
        for d in tqdm(folder_data, desc = "Unesting Folder Data", unit=" unested", colour="#2c8558"):
            folder_dict = d
            metadata_list = []
            metadata_list = self.walk_folder_structure(
                dict_obj=folder_dict,
                data_storage=metadata_list,
                parent_name='1')

        logger.debug('retrieved yaml folder files')
        logger.debug('folder metadata = %s', metadata_list)

        return metadata_list

    def walk_folder_structure(self,
                              dict_obj: dict,
                              data_storage: list,
                              parent_name: str):
        folder_name = dict_obj.get('name')

        try:
            if parent_name == 'Shared':
                logger.debug(
                    f'Creating folder Shared Child Folder "{folder_name}"')
                folder = self.create_folder(
                    folder_name=folder_name, parent_id='1')
                new_folder_id = folder.get('id')
            elif parent_name == '1':
                folder = self.sdk.folder(folder_id='1')
                new_folder_id = 'Shared'
            else:
                folder = self.create_folder(folder_name=folder_name,
                                            parent_id=parent_name)
                new_folder_id = folder.get('id')

            temp = {}
            temp['name'] = folder_name
            temp['old_folder_id'] = dict_obj.get('id')
            temp['new_folder_id'] = new_folder_id
            temp['content_metadata_id'] = folder.get('content_metadata_id')
            temp['team_edit'] = dict_obj.get('team_edit')
            temp['team_view'] = dict_obj.get('team_view')
            logger.debug('data_structure to be appended = %s', temp)
            data_storage.append(temp)
        except error.SDKError as e:
            logger.warn('error %s', e)
            logger.warn(
                'you have a duplicate folder called %s with the same parent, this is against best practice and LManage is ignoring it', folder_name)

        if isinstance(dict_obj.get('subfolder'), list):
            for subfolder in dict_obj.get('subfolder'):
                self.walk_folder_structure(subfolder, data_storage,
                                           parent_name=new_folder_id)

        return data_storage

    def create_folder(self,
                      folder_name: str,
                      parent_id: str,
                      ):
        logger.debug(f'Creating folder "{folder_name}"')
        folder = self.sdk.create_folder(
            body=models.CreateFolder(
                name=folder_name,
                parent_id=parent_id
            )
        )
        return folder

    def sync_folders(self, created_folder: list):
        all_folders = self.sdk.all_folders()
        created_folder_ids = [fobj.get('folder_id') for fobj in created_folder]
        all_folder_ids = []

        for folder in all_folders:
            if folder.is_personal:
                pass
            elif folder.parent_id is None:
                pass
            else:
                all_folder_ids.append(folder.id)

        for fids in all_folder_ids:
            if fids not in created_folder_ids:
                try:
                    self.sdk.delete_folder(folder_id=fids)
                    logger.debug(
                        'deleting folder %s to sync with yaml config', fids)

                except error.SDKError as InheritanceError:
                    logger.debug('root folder has been deleted so %s',
                                InheritanceError)
        return 'your folders are in sync with your yaml file'

    def create_folders(self):
        folder_metadata_list = []
        all_folders = self.get_all_folders()
        self.scour_folder()
        created_folder_metadata = self.unnest_folder_data(
            folder_data=self.folder_metadata)
        return created_folder_metadata

    def create_folder_mapping_dict(self, folder_metadata: list) -> dict:
        '''
        create a folder mapping dict with legacy folder id as it's key and new folder id as the value for use in content migration
        '''
        response = {}
        for folder_obj in folder_metadata:
            lid = folder_obj.get('old_folder_id')
            fid = folder_obj.get('new_folder_id')
            response[lid] = fid

        return response



from tqdm import tqdm
from looker_sdk import models40 as models
import coloredlogs
import logging
logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')

class Create_Dashboards():

    def __init__(self, sdk, folder_mapping, content_metadata) -> None:
        self.sdk = sdk
        self.folder_mapping = folder_mapping
        self.content_metadata = content_metadata

    
    def upload_dashboards(self) -> None:
        resp = []
        for dash in tqdm(self.content_metadata, desc = "Dashboard Upload", unit="dashboards", colour="#2c8558"):
            print(type(dash))
            t = dash.get('legacy_folder_id').get('folder_id')
            new_folder_id = self.folder_mapping.get(t)
            new_folder_id = new_folder_id if new_folder_id != 'Shared' else 1 
            body = models.WriteDashboardLookml(
                folder_id=new_folder_id, 
                lookml=dash['lookml']
            )
            # lookml = self.amend_lookml_str(dash['lookml'])
            temp = {}
            new_dash = self.sdk.import_dashboard_from_lookml(body=body)
            temp[dash['dashboard_id']] = new_dash.dashboard_id if new_dash.dashboard_id else dash['dashboard_slug']
            resp.append(temp)
        return resp


    def execute(self):
        mapping = self.upload_dashboards()
        return mapping


import logging
import coloredlogs
from yaspin import yaspin
from lmanage.utils import looker_object_constructors as loc
from looker_sdk import error
from tqdm import tqdm
from tenacity import retry, wait_random, wait_fixed, stop_after_attempt

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')
logging.getLogger("requests").setLevel(logging.WARNING)


class CaptureFolderConfig():
    ''' Class to read in folder metadata and unested_folder_data '''

    def __init__(self, sdk):
        self.sdk = sdk

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_folder_ancestors(self, dashboard_folder_id: str) -> dict:
        response = self.sdk.folder_ancestors(
            folder_id=dashboard_folder_id, fields="name")
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_all_folders(self) -> list:
        response = self.sdk.all_folders()
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_folder_metadata(self, folder_id: str) -> list:
        response = self.sdk.folder(folder_id)
        return response

    def get_all_folders(self) -> dict:
        '''retriving all Looker instance folder id and creating a dict of folder to root folder'''
        system_folder_id = ['1', '2', '3', '4', '5']
        trys = 0
        all_folder_metadata = None
        while all_folder_metadata is None:
            try:
                with yaspin().white.bold.shark.on_blue as sp:
                    sp.text = f"getting folder metadata"
                    all_folder_metadata = self.sdk.all_folders()
            except error.SDKError as e:
                logger.debug(e)
        all_folder_metadata = [
            folder.id for folder in all_folder_metadata if not folder.is_personal]
        folder_length = len(all_folder_metadata)

        folder_history = {}

        l = 0
        for dash in all_folder_metadata:
            l += 1
            if dash in list(folder_history.keys()):
                folder_root = folder_history.get(dash)
            else:
                folder_root = None
                trys = 0
                with yaspin().white.bold.shark.on_blue as sp:
                    sp.text = f"getting folder ancestors for folder {l} / {folder_length}"
                    folder_root = self.get_folder_ancestors(
                        dashboard_folder_id=dash)
                if not folder_root:
                    folder = self.get_folder_metadata(dash)
                    folder_history[dash] = [{'name': folder.name}]
                else:
                    folder_history[dash] = folder_root
        return folder_history

    @staticmethod
    def clean_folders(folder_list):
        '''removing standard system folders from lmanage.folder list'''
        system_folders = ['Users', 'Embed Users', 'Embed Groups']
        removal_folder_id = ['1', '2', '3', '4',  'lookml', '5']
        response = []
        folder_list = {k: v for k, v in folder_list.items()
                       if k not in removal_folder_id}
        cleaned_folders = {folder_id: root_name for folder_id,
                           root_name in folder_list.items() if root_name[0]['name'] not in system_folders}

        response = sorted(list(cleaned_folders.keys()), reverse=True)
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_all_content_metadata_accesses(self, cmi: int) -> dict:
        response = self.sdk.all_content_metadata_accesses(
            content_metadata_id=cmi)
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_group_metadata(self, gid: int) -> dict:
        response = self.sdk.group(
            group_id=gid)
        return response

    def get_content_access_metadata(self, cmi, root_folder: bool):
        r = []
        cmi_metadata = None
        if root_folder:
            cmi_metadata = self.get_all_content_metadata_accesses(cmi=1)
        else:
            cmi_metadata = self.get_all_content_metadata_accesses(cmi=cmi)

        for cmi in cmi_metadata:
            group_id = cmi.group_id
            if group_id is not None:
                permission_type = cmi.permission_type.value
                temp = {}
                group_meta = self.get_group_metadata(gid=group_id)
                temp[permission_type] = group_meta.get('name')
                r.append(temp)
            else:
                logger.info(
                    'no group permissions set on folder, ignoring individual permissions')
                pass
        return r

    def create_folder_objects(self, folder_list):
        '''creating folder objects (class in utils folder)'''
        response = {}
        folder_list = sorted(folder_list, key=int, reverse=True)
        for folder in tqdm(folder_list, desc="Folder Capturation", unit=" folders", colour="#2c8558"):
            f_metadata = self.get_folder_metadata(folder_id=folder)

            if f_metadata.name in ['Shared', 'Users']:
                folder_name = f_metadata.name
                f_metadata.name = '%s_' % folder_name
                logger.debug(f_metadata)

            if f_metadata.is_personal or f_metadata.is_personal_descendant or f_metadata.is_embed:
                logger.debug(
                    'folder %s  will be ignored as it\'s a personal folder or embed folder', f_metadata.name)
            else:
                content_metadata_id = f_metadata.get('content_metadata_id')
                a_list = self.get_content_access_metadata(
                    cmi=content_metadata_id, root_folder=False)

                created_folder_object = loc.LookerFolder(
                    id=folder, folder_metadata=f_metadata, access_list=a_list)
                logger.debug('capturing folder %s', f_metadata.get('name'))
                response[folder] = created_folder_object

        root_content_meta = self.get_content_access_metadata(
            cmi=1, root_folder=True)
        root_f_meta = self.get_folder_metadata(folder_id=str(1))

        root_folder = loc.LookerFolder(
            id='1', folder_metadata=root_f_meta, access_list=root_content_meta)
        response['1'] = root_folder
        return response

    @ staticmethod
    def get_highest_folders(folder_list):
        '''helper to isolate highest level folders and return as a list'''
        response = [
            folder for folder in folder_list if folder.parent_id == '1']
        return response

    @ staticmethod
    def get_subfolders(folder_list, parent_id):
        '''helper function to get all subfolders for a given parent'''
        response = []
        for folder in folder_list:
            if folder.parent_id == parent_id:
                response.append(folder)
        return response

    def create_nested_folder_objects(self, folder_list):
        '''create folder objects list with all children attached return
        full list of instance folder structure'''
        for f in folder_list:
            folder = folder_list.get(f)
            f_parent_id = folder.parent_id

            if f_parent_id in list(folder_list.keys()):
                parent_folder = folder_list.get(f_parent_id)
                parent_folder.add_child_folder(folder)
            else:
                pass

        # folder_list = [f for f in list(
        #     folder_list.values()) if f.parent_id == '1']
        folder_list = [f for f in list(folder_list.values()) if f.id == '1']

        return folder_list

    def execute(self):
        '''main function to do chain all the processes'''
        folders = self.get_all_folders()
        clean_folders = self.clean_folders(folder_list=folders)
        created_folder_list = self.create_folder_objects(
            folder_list=clean_folders)
        final_folder_list = self.create_nested_folder_objects(
            folder_list=created_folder_list)
        return folders, final_folder_list



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import coloredlogs
import logging
import click
from lmanage import capture_instance_permission_structure

logger = logging.getLogger(__name__)


@click.group(name='capturator')
def capturator():
    """Captures security settings for your looker instance"""
    pass


@capturator.command()
@ click.option("-i", "--ini-file",
               help="**OPTIONAL ** Specify API Credentials in an ini file, if no path is given program will assume these values are set as environmental variables as denoted at https: // github.com/looker-open-source/sdk-codegen  # environment-variable-configuration")
@ click.option("-yep", "--yaml-export-path",
               help="Where to save the yaml file to use for instance configuration")
@ click.option("-l", "--level",
               default='INFO',
               help="**OPTIONAL** Add the value 'DEBUG' to get a more verbose version of the returned stout text")
def capturator(**kwargs):
    """Captures security settings for your looker instance"""
    level = kwargs.get('level', 'INFO')
    coloredlogs.install(level=level, logger=logger)
    for k, v in kwargs.items():
        if {v} != None:
            logger.info(
                f'You have set {v} for your {k} variable')
        else:
            logger.debug(
                f'There is no value set for {k} please use the `--help` flag to see input parameters')
    capture_instance_permission_structure.main(**kwargs)


if __name__ == '__main__':
    capturator()



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import click
from lmanage.capturator import commands as cap_commands
from lmanage.configurator import commands as config_commands


@click.group()
@click.version_option()
def lmanage():
    pass


lmanage.add_command(cap_commands.capturator)
lmanage.add_command(config_commands.configurator)
if __name__ == '__main__':
    lmanage()



import logging
import time
import coloredlogs
from looker_sdk import models, error
from lmanage.utils import errorhandling
from tqdm import tqdm
from lmanage.utils.errorhandling import return_sleep_message

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


class CreateAndProvisionInstanceFolders():
    def __init__(self, folders, sdk):
        self.sdk = sdk
        self.instance_folder_metadata = folders

    def get_content_access_metadata(self, folder: dict) -> list:
        response = []

        folder_list = self.sdk.all_folders()
        folder_cmaid_lookup = {
            folder.name: folder.content_metadata_id for folder in folder_list}

        folder_name = folder.get('name')
        cmaid = folder_cmaid_lookup.get(folder_name)
        temp_dict = {}
        temp_dict['name'] = folder_name
        temp_dict['cmi'] = cmaid
        edit_group = folder.get('team_edit')
        view_group = folder.get('team_view')

        perms = []
        if isinstance(edit_group, list):
            for group in edit_group:
                group_dict = {}
                egmetadata = self.sdk.search_groups(name=group)
                group_dict['name'] = group
                group_dict['id'] = egmetadata[0].id
                group_dict['permission'] = 'edit'
                perms.append(group_dict)
        else:
            group_dict = {}
            group_dict['name'] = 'no_name'
            group_dict['id'] = 'no_id'
            group_dict['permission'] = 'no_permission'
            perms.append(group_dict)

        if isinstance(view_group, list):
            for group in view_group:
                group_dict = {}
                vgmetadata = self.sdk.search_groups(name=group)
                group_dict['name'] = group
                group_dict['id'] = vgmetadata[0].id
                group_dict['permission'] = 'view'
                perms.append(group_dict)
        else:
            group_dict = {}
            group_dict['name'] = 'no_name'
            group_dict['id'] = 'no_id'
            group_dict['permission'] = 'no_permission'
            perms.append(group_dict)

        temp_dict['group_permissions'] = perms
        response.append(temp_dict)
        return response

    def create_content_metadata_access(
        self,
            group_id: int,
            permission_input: str,
            content_metadata_id: int) -> dict:

        self.sdk.create_content_metadata_access(
            body=models.ContentMetaGroupUser(
                content_metadata_id=content_metadata_id,
                permission_type=permission_input,
                group_id=group_id
            )
        )
        folder = self.sdk.content_metadata(
            content_metadata_id=content_metadata_id).name
        group = self.sdk.search_groups(id=group_id)[0].name
        logger.debug(
            f'''--> Successfully permissioned group {group} with {permission_input} access, on folder {folder}.''')

    def check_existing_access(self,
                              content_metadata_id: int) -> dict:
        # check for existing access to the folder
        response = {
            access.group_id: access
            for access in self.sdk.all_content_metadata_accesses(
                content_metadata_id=content_metadata_id)}
        return response

    def check_folder_ancestors(self,
                               group_id: int,
                               cmaid: int):
        '''
        This method
        '''

        # getting folder id from content metadata number
        folder_id = self.sdk.content_metadata(
            content_metadata_id=cmaid).folder_id
        # returning the folder ancestors of the folder (i.e. all folders preceding)
        ancestors_list = self.sdk.folder_ancestors(folder_id=folder_id)
        # refining list to eliminate the Shared folder
        ancestors_list = [
            folder for folder in ancestors_list if folder.id != '1']

        # looping through folder ancestors checking their access corresponds with yaml access, if not update
        for ancestor in ancestors_list:
            ancestor_cmaid = ancestor.content_metadata_id
            folder_access = self.check_existing_access(
                content_metadata_id=ancestor_cmaid)
            # if this condition is met then do nothing
            if group_id not in folder_access:
                if self.check_folder_inheritance(
                        content_metadata_id=ancestor_cmaid):
                    # N.B. when you update folder inheritance you will create a content metadata access
                    # so recheck value to prevent trying to set an already set value
                    self.update_folder_inheritance(
                        cmaid=ancestor_cmaid, inheritance=False)
                    folder_access = self.check_existing_access(
                        content_metadata_id=ancestor_cmaid)
                    if folder_access:
                        pass
                    else:
                        self.create_content_metadata_access(
                            group_id=group_id,
                            permission_input='view',
                            content_metadata_id=ancestor_cmaid)
                        self.update_folder_inheritance(
                            cmaid=ancestor_cmaid, inheritance=True)
                else:
                    self.create_content_metadata_access(
                        group_id=group_id,
                        permission_input='view',
                        content_metadata_id=ancestor_cmaid)

    def check_folder_inheritance(self,
                                 content_metadata_id: int) -> bool:
        response = self.sdk.content_metadata(
            content_metadata_id=content_metadata_id)
        r = response.inherits
        return r

    def check_folder_inheritance_change(self,
                                        folder_input: dict) -> bool:
        check = []
        perms = folder_input['group_permissions']
        for permission in perms:
            if permission['permission'] == 'view':
                check.append('true')
            elif permission['permission'] == 'edit':
                check.append('true')
            else:
                check.append('false')

        if 'true' in check:
            return True
        else:
            return False

    def update_folder_inheritance(
            self,
            cmaid: int,
            inheritance: bool):
        # don't want to inherit access from lmanage.parent folders
        self.sdk.update_content_metadata(
            content_metadata_id=cmaid,
            body=models.WriteContentMeta(inherits=inheritance)
        )

    def add_content_access(
        self,
            cm_accesses: dict,
            cmaid: int,
            group_permissions: list) -> dict:

        for group in group_permissions:
            group_id = group.get('id')
            permission = group.get('permission')
            self.check_folder_ancestors(
                group_id=group_id,
                cmaid=cmaid
            )
            self.update_folder_inheritance(
                cmaid=cmaid,
                inheritance=False
            )
            group_name = self.sdk.search_groups(id=group_id)[0].name

            if group_id in cm_accesses.keys():
                current_access = cm_accesses.get(group_id)

                if current_access.permission_type == permission:
                    logger.debug(
                        f'''--> Group {group_name} already has access, to folder {folder_name}
                        no changes made.''')

                else:
                    try:
                        self.sdk.update_content_metadata_access(
                            content_metadata_access_id=current_access.id,
                            body=models.ContentMetaGroupUser(
                                content_metadata_id=cmaid,
                                permission_type=permission,
                                group_id=group_id
                            ))
                    except error.SDKError as foldererror:
                        logger.debug(foldererror)
                    folder_name = self.sdk.content_metadata(
                        content_metadata_id=cmaid).name
                    group_name = self.sdk.search_groups(id=group_id)[0].name

                    logger.debug(
                        f'--> Updating group id {group_name} permission type to {permission} on folder {folder_name}.')

            # no existing access
            # create from lmanage.scratch
            elif permission == 'no_permission':
                pass
            else:
                self.create_content_metadata_access(
                    content_metadata_id=cmaid,
                    permission_input=permission,
                    group_id=group_id
                )

    def provision_folders_with_group_access(self, content_access_metadata_list: list):

        for folder_tree in tqdm(content_access_metadata_list, desc="Folder Perms - Group Access", unit=" folders provisioned", colour="#2c8558"):

            for access_item in folder_tree:
                content_metadata_id = access_item["cmi"]

                self.update_folder_inheritance(
                    cmaid=content_metadata_id, inheritance=False)

                gp_permissions = access_item.get('group_permissions')

                self.sync_folder_permission(cmaid=content_metadata_id,
                                            gp_permissions=gp_permissions)

    def remove_content_access(self,
                              cm_accesses: dict):

        # remove all accesses
        for group_id in cm_accesses.keys():
            try:
                delete_cmi = cm_accesses.get(group_id).id
                self.sdk.delete_content_metadata_access(
                    content_metadata_access_id=delete_cmi)
            except error.SDKError as InheritanceError:
                logger.debug('''You have an inheritance error in your YAML file possibly 
                            around %s, skipping group_id %s''', delete_cmi, group_id)
                logger.debug(InheritanceError)

    def sync_folder_permission(
        self,
        cmaid: int,
            gp_permissions: list):

        # check for existing access to the folder
        content_metadata_accesses = self.check_existing_access(
            content_metadata_id=cmaid)

        # check group permissions and add back 1 by 1
        gp_permissions = [
            perms for perms in gp_permissions if perms.get('id') != 'no_id']

        logger.debug(
            'syncing folder permissions for content metadata id %s', cmaid)

        if not gp_permissions:
            self.update_folder_inheritance(
                cmaid=cmaid, inheritance=True)
            self.update_folder_inheritance(cmaid=cmaid, inheritance=False)
        else:
            # remove folder content accesses
            self.remove_content_access(
                cm_accesses=content_metadata_accesses)
            # check for existing access to the folder
            content_metadata_accesses = self.check_existing_access(
                content_metadata_id=cmaid)

            self.add_content_access(
                cm_accesses=content_metadata_accesses,
                cmaid=cmaid,
                group_permissions=gp_permissions)

    def remove_all_user_group(
        self,
            content_access_metadata_list: list):

        # remove parent Shared group instance access
        try:
            folder = '1'
            self.sdk.update_content_metadata_access(
                content_metadata_access_id=folder,
                body=models.ContentMetaGroupUser(
                    permission_type='view',
                    content_metadata_id=folder,
                    group_id=folder
                )
            )
        except error.SDKError:
            logger.debug('All Users group already configured')
        clean = list()
        for avt in content_access_metadata_list:
            temp = {}
            temp['name'] = avt[0].get('name')
            temp['cmi'] = avt[0].get('cmi')
            clean.append(temp)
        res_list = [i for n, i in enumerate(clean) if i not in clean[n + 1:]]

        for access_item in res_list:
            time.sleep(1)

            content_metadata_id = access_item["cmi"]

            # check for existing access to the folder
            content_metadata_accesses = {
                access.group_id: access
                for access in self.sdk.all_content_metadata_accesses(
                    content_metadata_id=content_metadata_id)}

            for id, value in content_metadata_accesses.items():
                logger.debug(f'Checking item {value.id}')
                if value.group_id == 1:
                    cmaid = value.id
                    self.sdk.delete_content_metadata_access(
                        content_metadata_access_id=cmaid)

    def execute(self):
        # CONFIGURE FOLDERS WITH EDIT AND VIEW ACCESS
        content_access_metadata = []
        for folder_obj in tqdm(self.instance_folder_metadata, desc="Folder Perms - Edit/View Access", unit=" folders", colour="#2c8558"):
            r = self.get_content_access_metadata(folder=folder_obj)
            content_access_metadata.append(r)

        # ADD AND SYNC CONTENT VIEW ACCESS WITH YAML
        self.provision_folders_with_group_access(
            content_access_metadata_list=content_access_metadata)
        self.remove_all_user_group(
            content_access_metadata_list=content_access_metadata)



import logging
import coloredlogs
from looker_sdk import models, error
from tqdm import tqdm

#logger = logging.getLogger(__name__)
#coloredlogs.install(level='DEBUG')


class CreateAndAssignUserAttributes():
    def __init__(self, user_attributes, sdk):
        self.user_attribute_metadata = user_attributes
        self.sdk = sdk

    def existing_user_attributes(self) -> dict:
        all_instance_ua = self.sdk.all_user_attributes()

        all_ua = {ua.name: ua.id for ua in all_instance_ua}
        return all_ua

    def create_user_attribute_if_not_exists(self):
        existing_ua = self.existing_user_attributes()
        yaml_user_attributes = self.user_attribute_metadata

        for ua in tqdm(yaml_user_attributes, desc = "User Attribute Creation", unit=" attributes", colour="#2c8558"):
            name = ua.get('name')
            if name in existing_ua.keys():
                logging.warning(
                    f'user attribute {name} already exists on this instance')
            else:
                datatype = ua.get('uatype')
                value_is_hidden = ua.get('hidden_value')
                user_view = ua.get('user_view')
                user_edit = ua.get('user_edit')

                try:
                    ua_permissions = models.WriteUserAttribute(
                        name=name,
                        label=name,
                        type=datatype,
                        value_is_hidden=value_is_hidden,
                        user_can_view=user_view,
                        user_can_edit=user_edit,
                        default_value=ua.get('default_value')
                    )
                    response = self.sdk.create_user_attribute(body=ua_permissions)
                    logging.info(f'created user attribute {response.label}')
                except error.SDKError as err:
                    logging.error('skipping permission because already exists')

    def sync_user_attributes(self):
        instance_ua = self.existing_user_attributes()
        config_ua = [ua.get('name') for ua in self.user_attribute_metadata]
        sys_default_ua = [
            'email',
            'first_name',
            'id',
            'google_user_id',
            'landing_page',
            'last_name',
            'number_format',
            'locale',
            'name'
        ]

        for ua in sys_default_ua:
            if ua in instance_ua:
                instance_ua.pop(ua, None)

        for ua_name in instance_ua.keys():
            if ua_name not in config_ua:
                ua_id = instance_ua.get(ua_name)
                self.sdk.delete_user_attribute(ua_id)
                logging.info(
                    f'''deleting ua {ua_name} because
                    it is not listed in the yaml config''')

    def add_group_values_to_ua(self):
        instance_ua = self.existing_user_attributes()
        all_instance_groups = self.sdk.all_groups()
        group_metadata = {
            group.name: group.id for group in all_instance_groups}
        yaml_user_attributes = self.user_attribute_metadata
        for ua in yaml_user_attributes:
            if len(ua.get('teams')) > 0:
                for team_val in ua.get('teams'):
                    group_id = group_metadata.get(list(team_val.keys())[0])

                    meta_value = list(team_val.values())[0]
                    ua_id = instance_ua.get(ua.get('name'))

                    params_to_add = models.UserAttributeGroupValue(
                        value=meta_value,
                        value_is_hidden=False
                    )

                    self.sdk.update_user_attribute_group_value(
                        group_id=group_id,
                        user_attribute_id=ua_id,
                        body=params_to_add)

    def execute(self):
        # CREATE NEW USER ATTRIBUTES
        self.create_user_attribute_if_not_exists()

        # DELETE ALL USER ATTRIBUTES THAT DON'T MATCH WITH YAML
        self.sync_user_attributes()

        # ADD VALUES TO INSCOPE USER ATTRIBUTES
        self.add_group_values_to_ua()



from tqdm import tqdm
import coloredlogs
import logging
logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')

class CleanInstanceContent():
    def __init__(self, sdk) -> None:
        self.sdk = sdk

    def empty_looker_dashboard_trash(self) -> None:
        trash_dash = self.sdk.search_dashboards(deleted=True)
        trash_dash_id_list = [dash.id for dash in trash_dash]
        for dash_id in tqdm(trash_dash_id_list, 'Emptying Dashboard Trash', unit='dashboards', colour="#2c8558"):
            self.sdk.delete_dashboard(dashboard_id=dash_id)
            logger.debug(
                f'cleaning the trash dashboard {dash_id} from instance')

    def empty_looker_look_trash(self) -> None:
        trash_look = self.sdk.search_looks(deleted=True)
        trash_look_id_list = [look.id for look in trash_look]
        for look_id in tqdm(trash_look_id_list, 'Emptying Look Trash', unit='Look', colour="#2c8558"):
            self.sdk.delete_look(look_id=look_id)
            logger.debug(
                f'cleaning the trash dashboard {look_id} from instance')
                
                
    def delete_boards(self, all_board: list) -> None:
        delete_board_list = [board.id for board in all_board]
        for board_id in tqdm(delete_board_list, 'Scrubbing Instance Boards', unit='boards', colour="#2c8558"):
            self.sdk.delete_board(board_id=board_id)
            logger.debug(
                f'cleaning the trash board {board_id} from instance')



    def delete_instance_dash(self, dashboards: list) -> None:
        delete_id_list = [dash.id for dash in dashboards]
        for dash_id in tqdm(delete_id_list, 'Scrubbing Instance Dashboards', unit='dashboards', colour="#2c8558"):
            self.sdk.delete_dashboard(dashboard_id=dash_id)
            logger.debug(
                f'cleaning the trash dashboard {dash_id} from instance')

    def delete_instance_look(self, looks: list) -> None:
        delete_id_list = [look.id for look in looks]
        for look_id in tqdm(delete_id_list, 'Scrubbing Instance Looks', unit='looks', colour="#2c8558"):
            self.sdk.delete_look(look_id=look_id)
            logger.debug(
                f'cleaning the trash dashboard {look_id} from instance')

    def execute(self):
        all_dash_list = self.sdk.all_dashboards()
        all_look_list = self.sdk.all_looks()
        all_board_list = self.sdk.all_boards()
        self.delete_instance_dash(dashboards=all_dash_list)
        self.delete_instance_look(looks=all_look_list   )
        self.delete_boards(all_board=all_board_list)
        self.empty_looker_dashboard_trash()
        self.empty_looker_look_trash()



import logging
import coloredlogs
from looker_sdk import models, error
from lmanage.utils.errorhandling import return_error_message

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


class GetInstanceGroups():
    def __init__(self, sdk) -> None:
        self.sdk = sdk

    def extract_group_names(self):
        capture_groups = self.sdk.all_groups()
        group_names = [group.name for group in capture_groups]

        return group_names

    def extract_folder_names(self):
        folders = self.sdk.all_folders()
        test = {}
        for folder in folders:
            print('high', folder.name)
            if folder.parent_id is not None:
                print('low', folder.name)

                test[folder.name] = folder.parent_id
            else:
                print('test', folder.name)

        # for x, y in test.items():

        return test

    def create_group_if_not_exists(self,
                                   sdk,
                                   group_name: str) -> dict:
        """ Create a Looker Group and add Group attributes

        :group_name: Name of a Looker group to create.
        :rtype: Looker Group object.
        """
        # get group if exists
        try:
            logger.info(f'Creating group "{group_name}"')
            group = sdk.create_group(
                body=models.WriteGroup(
                    can_add_to_content_metadata=True,
                    name=group_name
                )
            )
            return group
        except error.SDKError as grouperr:
            err_msg = return_error_message(grouperr)
            logger.info('you have an error in your group; error = %s', err_msg)
            logger.debug(grouperr)
            group = sdk.search_groups(name=group_name)
            return group[0]

    def get_instance_group_metadata(self,
                                    sdk,
                                    unique_group_list: list) -> list:
        group_metadata = []

        for group_name in unique_group_list:
            group = self.create_group_if_not_exists(sdk, group_name)
            temp = {}
            temp['group_id'] = group.id
            temp['group_name'] = group.name
            group_metadata.append(temp)

        return group_metadata

    def sync_groups(self,
                    sdk,
                    group_name_list: list) -> str:

        all_groups = sdk.all_groups()
        group_dict = {group.name: group.id for group in all_groups}
        # Deleting Standard Groups
        del group_dict['All Users']

        for group_name in group_dict.keys():
            if group_name not in group_name_list:
                sdk.delete_group(group_id=group_dict[group_name])
                logger.info(
                    f'deleting group {group_name} to sync with yaml config')

        return 'your groups are in sync with your yaml file'

    def execute(self):
        team_list = []
        # extracting user attribute teams
        self.extract_teams(container=self.user_attribute_metadata,
                           data_storage=team_list)
        # extracting role based teams
        self.extract_teams(
            container=self.role_metadata, data_storage=team_list)
        # extract nested folder access teams
        self.extract_folder_teams(
            container=self.folder_metadata, data_storage=team_list)

        team_list = list(set(team_list))

        # create all the groups
        self.get_instance_group_metadata(
            sdk=self.sdk, unique_group_list=team_list)



import logging
import time
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')

class CreateFolderOutput():
    def __init__(self, folder_list):
        self.folder_list = folder_list

    def create_folder_structure(self, folder_obj: object, data_storage: list):
        temp = {}
        temp['name'] = folder_obj.name
        temp['permissions'] = folder_obj.content_access_list
        temp['subfolder'] = folder_obj.children
        data_storage.append(temp)

        if isinstance(folder_obj.children, list):
            for subfolder in folder_obj.children:
                self.create_folder_structure(subfolder, data_storage)

        return data_storage

    def make_dict(self):
        x = []
        f = self.folder_list

        for folders in f:
            data_storage = []
            x = self.create_folder_structure(
                folder_obj=folders, data_storage=data_storage)
        return x

    def execute(self):
        '''
        goal: extract all the folders into the appropriate markdown folder structure
        '''
        x = self.make_dict()
        return x



import logging
import time
import coloredlogs
import lmanage.utils.looker_object_constructors as loc
from lmanage.utils.errorhandling import return_sleep_message
from tqdm import tqdm
from yaspin import yaspin

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')

class CaptureLookObject():
    def __init__(self, sdk, folder_root: dict):
        self.sdk = sdk
        self.folder_root = folder_root

    def all_looks(self):
        system_folders = ['Users','Embed Users','Embed Groups']
        all_look_meta = None
        with yaspin().white.bold.shark.on_blue as sp:
            sp.text="getting all system look metadata (can take a while)"
            all_look_meta = self.sdk.all_looks(fields='id,folder')
            
        scrub_looks = {}
        for look in all_look_meta:
            folder_root = self.folder_root.get(look.folder.id,[{'name': 'Users'}])[0]['name']
            if look.folder.id in list(self.folder_root.keys()) and folder_root not in system_folders:
                scrub_looks[look.id] = look.folder.id 
            else:
                continue

        return scrub_looks

    def get_look_metadata(self, look_id: str) -> dict:
        look_meta = None
        trys = 0
        while look_meta is None:
            try:
                look_meta = self.sdk.look(look_id=look_id)
            except:
                return_sleep_message
        return look_meta

    def clean_query_obj(self, query_metadata: dict) -> dict:
        metadata_keep_keys = [
            "model",
            "view",
            "fields",
            "pivots",
            "fill_fields",
            "filters",
            "filter_expression",
            "sorts",
            "limit",
            "column_limit",
            "total",
            "row_total",
            "subtotals",
            "vis_config",
            "filter_config",
            "visible_ui_sections",
            "dynamic_fields",
            "query_timezone"]
        restricted_look_metadata = dict(
            (k, query_metadata[k]) for k in metadata_keep_keys)
        return restricted_look_metadata


    def execute(self):
        '''
        1. get all the looks and extract the id's
        2. iterate through the looks and extract metadata, only keep necessary fields to make a query for look transference
        3. create custom look object and add to list
        '''
        all_look_data = self.all_looks()
        looks = []
        content = 1
        for look in tqdm(all_look_data, desc = "Look Capture", unit=" looks", colour="#2c8558"):
            total = len(all_look_data)
            lmetadata = self.get_look_metadata(look_id=look)
            query_object = lmetadata.query.__dict__
            nq_obj = self.clean_query_obj(query_metadata=query_object)

            legacy_folder = lmetadata.folder_id
            look_id = lmetadata.id
            title = lmetadata.title
            look_obj = loc.LookObject(
                query_obj=nq_obj,
                description=lmetadata.description,
                legacy_folder_id=legacy_folder,
                look_id=look_id,
                title=title)
            looks.append(look_obj)
            content += 1
        return looks



import logging
import coloredlogs
import ruamel.yaml

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)

yaml = ruamel.yaml.YAML()


@yaml.register_class
class LookerFolder():
    def __init__(self, id, folder_metadata, access_list):
        self.parent_id = folder_metadata.get('parent_id')
        self.id = id
        self.name = folder_metadata.get('name')
        self.subfolder = []
        self.content_metadata_id = folder_metadata.get('content_metadata_id')
        self.team_edit = self.breakup_access_list(
            access_list=access_list, access_type='edit')
        self.team_view = self.breakup_access_list(
            access_list=access_list, access_type='view')

    def add_child_folder(self, ref):
        self.subfolder.append(ref)

    def breakup_access_list(self, access_list, access_type):
        response = []
        for access in access_list:
            team = access.get(access_type, None)
            if team is not None:
                response.append(team)
        return response


class LookerPermissionSet():
    def __init__(self, permissions, name):
        self.permissions = permissions
        self.name = name


class LookerUserAttribute():
    def __init__(self, teams_val: dict, name: str, uatype: bool, hidden_value: bool, user_view, user_edit, default_value) -> object:
        self.name = name
        self.uatype = uatype
        self.hidden_value = hidden_value
        self.user_view = str(user_view)
        self.user_edit = str(user_edit)
        self.default_value = default_value
        self.teams = teams_val


class LookerModelSet():
    def __init__(self, models, name):
        self.models = models
        self.name = name


class LookerRoles():
    def __init__(self, permission_set, model_set, teams, name):
        self.permission_set = permission_set
        self.model_set = model_set
        self.teams = teams
        self.name = name


class LookerGroup():
    def __init__(self, id, group_metadata):
        self.name = group_metadata.name
        self.id = id
        self.children = []


class LookObject():
    def __init__(self, description, query_obj, title, legacy_folder_id, look_id):
        self.legacy_folder_id = legacy_folder_id
        self.look_id = look_id
        self.title = title
        self.query_obj = query_obj
        self.description = description


class DashboardObject():
    def __init__(self, legacy_folder_id, lookml, dashboard_id, dashboard_slug) -> None:
        self.legacy_folder_id = legacy_folder_id
        self.lookml = lookml
        self.dashboard_id = dashboard_id
        self.dashboard_slug = dashboard_slug


class BoardObject():
    def __init__(self, content_metadata_id, section_order, title, primary_homepage, board_sections, description) -> None:
        self.content_metadata_id = content_metadata_id
        self.section_order = section_order
        self.title = title
        self.primary_homepage = primary_homepage
        self.board_sections = board_sections
        self.description = description



import logging
import coloredlogs
import looker_sdk
from looker_sdk import models
from role_config import CreateRoleBase

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class CreateInstanceRoles(CreateRoleBase):
    def __init__(self, roles, sdk):
        self.role_metadata = roles
        self.sdk = sdk

    def create_permission_lookup(self):
        all_permission_sets = self.get_all_permission_sets()
        instance_permission_set_dict = {
            perm.name: perm.id for perm in all_permission_sets}
        return instance_permission_set_dict

    def create_model_lookup(self):
        all_model_sets = self.get_all_model_sets()
        instance_model_set_dict = {
            model.name: model.id for model in all_model_sets}
        return instance_model_set_dict

    def create_allgroup_lookup(self):
        all_groups = self.sdk.all_groups()
        group_dict = {group.name: group.id for group in all_groups}
        return group_dict

    def create_allrole_lookup(self):
        all_roles = self.sdk.all_roles()
        role_dict = {role.name: role.id for role in all_roles}
        return role_dict

    def create_instance_roles(self):
        model_lookup = self.create_model_lookup()
        permission_lookup = self.create_permission_lookup()

        role_output = []

        for role_name, metadata in self.role_metadata.items():
            model_set_id = model_lookup.get(metadata.get('model_set'))
            permission_set_id = permission_lookup.get(
                metadata.get('permission_set'))

            body = models.WriteRole(
                name=role_name,
                permission_set_id=permission_set_id,
                model_set_id=model_set_id
            )
            try:
                role = self.sdk.create_role(
                    body=body
                )
            except looker_sdk.error.SDKError as roleerror:
                logger.debug(roleerror)
                role_id = self.sdk.search_roles(name=role_name)[0].id
                role = self.sdk.update_role(role_id=role_id, body=body)
            temp = {}
            temp['role_id'] = role.id
            temp['role_name'] = role_name
            role_output.append(temp)
        logger.info(role_output)
        return role_output

    def set_role(self, role_id: str, group_id: list) -> str:
        try:
            self.sdk.set_role_groups(role_id, group_id)
            return logger.info(f'attributing {group_id} permissions on instance')
        except looker_sdk.error.SDKError:
            return logger.info('something went wrong')

    def attribute_instance_roles(self, created_role_metadata: list):
        role_lookup = self.create_allrole_lookup()
        group_lookup = self.create_allgroup_lookup()

        for role_name, metadata in self.role_metadata.items():
            teams = metadata.get('team')
            role_id = role_lookup.get(role_name)
            group_id_list = []
            for team in teams:
                group_id = group_lookup.get(team)
                group_id_list.append(group_id)
            self.set_role(role_id=role_id, group_id=group_id_list)

    def sync_roles(self):
        all_role_lookup = self.create_allrole_lookup()
        all_role_lookup.pop('Admin')
        yaml_role = [role for role in self.role_metadata]
        logger.debug(all_role_lookup)

        for role_name in all_role_lookup.keys():
            if role_name not in yaml_role:
                role_id = self.sdk.search_roles(name=role_name)[0].id
                self.sdk.delete_role(role_id=role_id)

    def execute(self):
        created_roles = self.create_instance_roles()
        self.attribute_instance_roles(created_role_metadata=created_roles)
        self.sync_roles()



from looker_sdk import models, error
import logging
import coloredlogs
from lmanage.utils import errorhandling as eh
from tqdm import tqdm

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class CreateRoleBase():
    def __init__(self, permissions, model_sets, sdk):
        self.permission_set_metadata = permissions
        self.model_set_metadata = model_sets
        self.sdk = sdk

    def create_permission_set(self, sdk, permission_set_dict: dict):

        final_response = []
        for permission in tqdm(permission_set_dict, desc = "Creating Permission Sets", unit=" perm sets", colour="#2c8558"):
            permission_set_name = permission.get('name')
            if permission_set_name == 'Admin':
                pass
            else:
                permissions = permission.get('permissions')
                body = models.WritePermissionSet(
                    name=permission_set_name.lower(),
                    permissions=permissions
                )
                try:
                    perm = sdk.create_permission_set(
                        body=body
                    )
                except error.SDKError as permerr:
                    logger.debug(permerr)
                    err_msg = eh.return_error_message(permerr)
                    logger.debug(permerr)
                    logger.debug(
                        'you have a warning permission set %s, warning = %s already exists on this instance', permission_set_name, err_msg)

                    perm = sdk.search_permission_sets(
                        name=permission_set_name)[0]
                    pid = perm.id
                    perm = sdk.update_permission_set(
                        permission_set_id=pid, body=body)

                temp = {}
                temp['name'] = perm.name
                temp['pid'] = perm.id
                final_response.append(temp)

        logger.debug(final_response)
        return final_response

    def sync_permission_set(self, sdk, all_perms, permission_set_dict: dict):

        permissions_dict = {p.name: p.id for p in all_perms}
        permissions_dict.pop('Admin')
        yaml_permissions = [permission.get('name').lower()
                            for permission in permission_set_dict]
        logger.debug('yaml permissions configured = %s', yaml_permissions)
        logger.debug('existing permsissions = %s', permissions_dict.keys())

        for permission_set_name in permissions_dict.keys():

            if permission_set_name.lower() not in yaml_permissions:
                permission_id = permissions_dict.get(permission_set_name)
                sdk.delete_permission_set(permission_set_id=permission_id)

    def create_model_set(self, sdk, model_set_dict: list) -> list:
        final_response = []
        model_set_dict = eh.dedup_list_of_dicts(model_set_dict)
        for model in tqdm(model_set_dict, desc = "Creating Model Sets", unit=" model sets", colour="#2c8558"):
            model_set_name = model.get('name')
            attributed_models = model.get('models')
            body = models.WriteModelSet(
                name=model_set_name.lower(), models=attributed_models)
            try:
                model = sdk.create_model_set(body=body)
                temp = {}
                final_response.append(temp)
            except error.SDKError as modelerror:
                err_msg = eh.return_error_message(modelerror)
                logger.debug(
                    'you have a warning in creating model set %s, warning = %s', model_set_name, err_msg)
                logger.debug(modelerror)
                model = sdk.search_model_sets(name=model_set_name)[0]
                model_set_id = model.id
                try:
                    model = sdk.update_model_set(
                        model_set_id=model_set_id, body=body)

                    temp = {}
                    temp['model_set_name'] = model.name
                    temp['model_set_id'] = model.id
                    final_response.append(temp)
                except error.SDKError as txt:
                    err_msg = eh.return_error_message(txt)
                    logger.debug(
                        'The model %s, has encountered a warning, warning = %s', model_set_name, err_msg)
                    temp = {}
                    temp['model_set_name'] = model.name
                    temp['model_set_id'] = model.id
                    final_response.append(temp)

        logger.debug(final_response)
        return final_response

    def sync_model_set(self, sdk, all_models, model_set_list: list):
        model_sets_dict = {p.name: p.id for p in all_models}
        model_sets_dict.pop('All')
        yaml_model = [model.get('name').lower() for model in model_set_list]

        for model_set_name in model_sets_dict.keys():

            if model_set_name.lower() not in yaml_model:
                model_id = model_sets_dict.get(model_set_name)
                sdk.delete_model_set(model_set_id=model_id)

    def get_all_permission_sets(self):
        sdk = self.sdk
        r = sdk.all_permission_sets()
        return r

    def get_all_model_sets(self):
        sdk = self.sdk
        r = sdk.all_model_sets()
        return r

    def execute(self):
        # Creating permission sets if they don't exits already on instance
        self.create_permission_set(
            sdk=self.sdk, permission_set_dict=self.permission_set_metadata)
        # Creating model set if they don't exist on Looker instance
        self.create_model_set(
            sdk=self.sdk, model_set_dict=self.model_set_metadata)

        # Syncing Permission and Model sets with Yaml file.
        self.sync_permission_set(
            sdk=self.sdk, all_perms=self.get_all_permission_sets(), permission_set_dict=self.permission_set_metadata)
        self.sync_model_set(sdk=self.sdk, all_models=self.get_all_model_sets(
        ), model_set_list=self.model_set_metadata)



import logging
import coloredlogs
import itertools
from lmanage.utils.looker_object_constructors import LookerPermissionSet, LookerModelSet, LookerRoles
from lmanage.utils import errorhandling
from tqdm import tqdm

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class ExtractRoleInfo():
    def __init__(self, sdk):
        self.sdk = sdk
        self.role_base = self.get_all_roles()

    def get_all_roles(self):
        sdk = self.sdk
        response = sdk.all_roles()
        for role in enumerate(response):

            if role[1].name == 'Admin':
                response.pop(role[0])

        return response

    def create_list_of_permission_sets(self):
        response = []
        for role in self.role_base:
            if role.permission_set is None:
                raise Exception(f'role name {role.name} has no permission_set, please add one to be captured or delete the role')
            temp = {}
            temp['name'] = role.permission_set.name
            temp['permissions'] = role.permission_set.permissions
            response.append(temp)
        return response

    def extract_permission_sets(self):
        response = []
        p_list = errorhandling.dedup_list_of_dicts(
            self.create_list_of_permission_sets())
        for role in p_list:
            logger.debug(role)
            perm_set = LookerPermissionSet(
                name=role.get('name'),
                permissions=role.get('permissions'))
            response.append(perm_set)

        return response

    def create_list_of_roles(self):
        response = []
        for role in self.role_base:
            temp = {}
            temp['name'] = role.model_set.name
            temp['models'] = role.model_set.models
            response.append(temp)
        return response

    def extract_model_sets(self):
        response = []
        role_list = errorhandling.dedup_list_of_dicts(
            self.create_list_of_roles())
        for role in role_list:
            model_set = LookerModelSet(
                models=role.get('models'), name=role.get('name'))
            response.append(model_set)
        return response

    def extract_role_info(self):
        response = []
        for role in tqdm(self.role_base, desc = "Role Capture", unit=" roles", colour="#2c8558"):
            groups = None
            trys = 0
            while groups is None:
                trys += 1
                try:
                    groups = self.sdk.role_groups(role_id=role.id)
                except:
                    errorhandling.return_sleep_message(call_number=trys)
            role_groups = [group.name for group in groups]

            lookerrole = LookerRoles(permission_set=role.permission_set.name,
                                     model_set=role.model_set.name, teams=role_groups, name=role.name)
            response.append(lookerrole)

        return response



import logging
import json
import coloredlogs
from looker_sdk import models, error
from lmanage.configurator.user_group_configuration.role_config import CreateRoleBase
from lmanage.utils.errorhandling import return_error_message
from lmanage.utils.errorhandling import return_sleep_message
from tqdm import tqdm

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class CreateInstanceRoles(CreateRoleBase):
    def __init__(self, roles, sdk):
        self.role_metadata = roles
        self.sdk = sdk

    def create_permission_lookup(self):
        all_permission_sets = self.get_all_permission_sets()
        instance_permission_set_dict = {
            perm.name: perm.id for perm in all_permission_sets}
        return instance_permission_set_dict

    def create_model_lookup(self):
        all_model_sets = self.get_all_model_sets()
        instance_model_set_dict = {
            model.name: model.id for model in all_model_sets}
        return instance_model_set_dict

    def create_allgroup_lookup(self):
        all_groups = self.sdk.all_groups()
        group_dict = {group.name: group.id for group in all_groups}
        return group_dict

    def create_allrole_lookup(self):
        all_roles = self.sdk.all_roles()
        role_dict = {role.name: role.id for role in all_roles}
        return role_dict

    def create_instance_roles(self):
        model_lookup = self.create_model_lookup()
        permission_lookup = self.create_permission_lookup()

        role_output = []

        for r_metadata in tqdm(self.role_metadata, desc = "User Role Creation", unit=" user roles", colour="#2c8558"):
            role_name = r_metadata.get('name')
            model_set_id = model_lookup.get(
                r_metadata.get('model_set') if r_metadata.get('model_set') == 'All' else r_metadata.get('model_set').lower())
            permission_set_id = permission_lookup.get(
                r_metadata.get('permission_set').lower())
            if role_name != 'Admin':
                body = models.WriteRole(
                    name=role_name,
                    permission_set_id=str(permission_set_id),
                    model_set_id=str(model_set_id)
                )
                try:
                    role = self.sdk.create_role(
                        body=body
                    )

                except error.SDKError as roleerror:
                    err_msg = return_error_message(roleerror)
                    logger.debug(
                        'You have hit a warning creating your role \'%s\'; warning = %s', role_name, err_msg)
                    role_id = self.sdk.search_roles(name=role_name)[0].id
                    role = self.sdk.update_role(
                        role_id=str(role_id), body=body)
        
                temp = {}
                temp['role_id'] = role.id
                temp['role_name'] = role_name
                role_output.append(temp)
            else:
                pass
        logger.debug(role_output)
        return role_output

    def set_role(self, role_id: str, group_id: list) -> str:
        try:
            self.sdk.set_role_groups(role_id, group_id)
            return logger.debug(f'attributing {group_id} permissions on instance')
        except error.SDKError as role_err:
            err_msg = return_error_message(role_err)
            logger.debug(
                'You have hit a warning setting your role, warning = %s', err_msg)
            logger.debug(role_err)

    def attribute_instance_roles(self, created_role_metadata: list):
        role_lookup = self.create_allrole_lookup()
        group_lookup = self.create_allgroup_lookup()

        for r_metadata in tqdm(self.role_metadata, desc = "Instance Role Attribution", unit=" roles", colour="#2c8558"):
            role_name = r_metadata.get('name')
            teams = r_metadata.get('teams')
            role_id = role_lookup.get(role_name)
            group_id_list = []
            for team in teams:
                group_id = group_lookup.get(team)
                group_id_list.append(group_id)
            self.set_role(role_id=role_id, group_id=group_id_list)

    def sync_roles(self):
        all_role_lookup = self.create_allrole_lookup()
        all_role_lookup.pop('Admin')
        yaml_role = [role.get('name') for role in self.role_metadata]

        for role_name in all_role_lookup.keys():
            if role_name not in yaml_role:
                role_id = all_role_lookup.get(role_name)
                self.sdk.delete_role(role_id=role_id)

    def execute(self):
        created_roles = self.create_instance_roles()
        self.attribute_instance_roles(created_role_metadata=created_roles)
        self.sync_roles()



import logging
import coloredlogs
import looker_sdk
from lmanage.configurator.user_attribute_configuration import create_ua_permissions as cuap
from lmanage.configurator.folder_configuration import folder_config as fc, create_folder_permissions as cfp, create_folders as cf
from lmanage.configurator.user_group_configuration import role_config as rc, group_config as gc, user_permission as up
from lmanage.configurator.content_configuration import create_looks as cl, create_dashboards as cd, create_content_prep as ccp, create_boards as cb
from lmanage.utils import parse_yaml as py, errorhandling as eh


logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def main(**kwargs):
    div = '----------------------------------------------------------------------------------------'

    ini_file = kwargs.get("ini_file")
    yaml_path = kwargs.get("yaml_config_path")
    logger.info(div)
    #logger.info('parsing yaml file')
    yaml_split = yaml_path.split('.')
    settings_yaml = py.Yaml(yaml_path=f'{yaml_split[0]}.yaml')
    content_yaml = py.Yaml(yaml_path=f'{yaml_split[0]}_content.yaml')

    if ini_file:
        sdk = looker_sdk.init40(config_file=ini_file)
    else:
        sdk = looker_sdk.init40()
    
    if eh.user_authentication_test(sdk=sdk):
        logger.info('User is successfully authenticated to the API')
    else:
        raise Exception("User is not successfully authenticated please verify credentials")

    folder_metadata = settings_yaml.get_folder_metadata()
    if not folder_metadata:
        logger.warn(
            f'no folder_metadata specified please check your yaml file at {yaml_split[0]}.yaml')
    permission_set_metadata = settings_yaml.get_permission_metadata()
    if not permission_set_metadata:
        logger.warn(
            f'no permission_set_metadata specified please check your yaml file at {yaml_split[0]}.yaml')
    model_set_metadata = settings_yaml.get_model_set_metadata()
    if not model_set_metadata:
        logger.warn(
            f'no model_set_metadata specified please check your yaml file at {yaml_split[0]}.yaml')
    role_metadata = settings_yaml.get_role_metadata()
    if not role_metadata:
        logger.warn(
            f'no role_metadata specified please check your yaml file at {yaml_split[0]}.yaml')
    user_attribute_metadata = settings_yaml.get_user_attribute_metadata()
    if not user_attribute_metadata:
        logger.warn(
            f'no user_attribute_metadata specified please check your yaml file at {yaml_split[0]}.yaml')

    look_metadata = content_yaml.get_look_metadata()
    if not look_metadata:
        logger.warn(
            f'no look_metadata specified please check your yaml file at {yaml_split[0]}_content.yaml')

    dash_metadata = content_yaml.get_dash_metadata()
    if not dash_metadata:
        logger.warn(
            f'no dash_metadata specified please check your yaml file at {yaml_split[0]}_content.yaml')

    board_metadata = content_yaml.get_board_metadata()
    if not board_metadata:
        logger.warn(
            f'no board_metadata specified please check your yaml file at {yaml_split[0]}_content.yaml')


################################################################
# Role Config ################################################
################################################################
    # Create Permission and Model Sets
    rc.CreateRoleBase(permissions=permission_set_metadata,
                      model_sets=model_set_metadata, sdk=sdk).execute()

# ###############################################################
# Folder Config ################################################
###############################################################
    # CREATE NEW FOLDERS
    folder_objects = cf.CreateInstanceFolders(
        folder_metadata=folder_metadata, sdk=sdk)
    created_folder_metadata = folder_objects.create_folders()

    # CREATE FOLDER TO FOLDER Dict
    folder_mapping_obj = folder_objects.create_folder_mapping_dict(
        folder_metadata=created_folder_metadata)


################################################################
# Group Config ################################################
################################################################
    # CREATE NEW GROUPS FROM YAML FILE TEAM VALUES
    gc.CreateInstanceGroups(
        folders=created_folder_metadata, user_attributes=user_attribute_metadata, roles=role_metadata, sdk=sdk).execute()

################################################################
# Folder Provision Config ################################################
################################################################
    # CREATE NEW GROUPS FROM YAML FILE TEAM VALUES

    cfp.CreateAndProvisionInstanceFolders(
        folders=created_folder_metadata, sdk=sdk).execute()

################################################################
# Role Config ################################################
################################################################
    up.CreateInstanceRoles(roles=role_metadata, sdk=sdk).execute()

###############################################################
# User Attribute Config #######################################
###############################################################
    # FIND UNIQUE USER ATTRIBUTES AND ATTRIBUTE TO TEAM
    cuap.CreateAndAssignUserAttributes(
        user_attributes=user_attribute_metadata, sdk=sdk).execute()

###############################################################
# Content Transport Config #######################################
###############################################################
    # EMPTY TRASH CAN OF ALL DELETED CONTENT
    ccp.CleanInstanceContent(sdk=sdk).execute()
    
    # FIND LOOKS AND REMAKE THEM
    look_creator = cl.CreateInstanceLooks(
        folder_mapping=folder_mapping_obj, sdk=sdk, content_metadata=look_metadata)
    look_mapping_dict = look_creator.execute()

    # Find DASHBOARDS AND REMAKE THEM
    dash_creator = cd.Create_Dashboards(
        sdk=sdk, folder_mapping=folder_mapping_obj, content_metadata=dash_metadata)
    content_mapping_dict = dash_creator.execute()

    # REMAKE BOARDS
    # board_creator = cb.Create_Boards(sdk=sdk,board_metadata=board_metadata, dashboard_mapping=dash_creator, look_mapping=look_creator)
    # board_creator.execute()

    logger.info('lmanage has finished configuring your Looker instance!')


if __name__ == "__main__":
    YP = ('/usr/local/google/home/hugoselbie/code_sample/py/'
          'lmanage/tests/example_yamls/dev_output.yaml')
    IP = ('/usr/local/google/home/hugoselbie/code_sample/py/ini/dev.ini')

    main(
        ini_file=IP,
        yaml_config_path=YP)



from tqdm import tqdm
import logging
from lmanage.utils.errorhandling import return_sleep_message
from lmanage.utils.looker_object_constructors import DashboardObject
import coloredlogs
from yaspin import yaspin

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)

class CaptureDashboards():
    def __init__(self, sdk, folder_root: dict):
        self.sdk = sdk
        self.folder_root = folder_root

    def get_all_dashboards(self) -> dict:
        system_folders = ['Users','Embed Users','Embed Groups', 'LookML Dashboards']
        scrub_dashboards = {}
        with yaspin().white.bold.shark.on_blue as sp:
            sp.text="getting all system dashboard metadata (can take a while)"
            all_dashboards = self.sdk.all_dashboards(fields="id,folder, slug")
        
        for dash in all_dashboards:
            if dash.folder.id == 'lookml':
                continue
            else:
                folder_root = self.folder_root.get(dash.folder.id, [{'name':'Users'}])[0]['name']
                if dash.folder.id in list(self.folder_root.keys()) and folder_root not in system_folders:
                    scrub_dashboards[dash.id] = {}
                    scrub_dashboards[dash.id]['folder_id'] = dash.folder.id
                    scrub_dashboards[dash.id]['slug'] = dash.slug 
        return scrub_dashboards

    def get_dashboard_lookml(self, all_dashboards: dict) -> list:
        #logging.info("Beginning Dashboard Capture:")
        response = []
  
        for dash_id in tqdm(all_dashboards, desc = "Dashboard Capture", unit=" dashboards", colour="#2c8558"):
        
            lookml = None
            trys = 0
            if "::" in dash_id:
                continue
            else:
                while lookml is None:
                    trys += 1
                    try:
                        lookml = self.sdk.dashboard_lookml(dashboard_id=dash_id)
                    except:
                        return_sleep_message(call_number=trys, quiet=True)
                
                captured_dashboard = DashboardObject(
                    legacy_folder_id=all_dashboards.get(dash_id),
                    lookml=lookml.lookml,
                    dashboard_id=dash_id,
                    dashboard_slug=all_dashboards.get(dash_id).get('slug'))
                response.append(captured_dashboard)
        return response

    def execute(self):
        all_dashboards = self.get_all_dashboards()
        captured_dash = self.get_dashboard_lookml(all_dashboards)
        return captured_dash